"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("tns-core-modules/file-system");
var view_1 = require("tns-core-modules/ui/core/view");
var file_system_1 = require("tns-core-modules/file-system");
var observable_1 = require("tns-core-modules/data/observable/observable");
var WKNavigationDelegateImpl = (function (_super) {
    __extends(WKNavigationDelegateImpl, _super);
    function WKNavigationDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    WKNavigationDelegateImpl.initWithOwner = function (owner) {
        var handler = WKNavigationDelegateImpl.new();
        handler._owner = owner;
        return handler;
    };
    WKNavigationDelegateImpl.prototype.webViewDidFinishNavigation = function (webView, navigation) {
        var owner = this._owner.get();
        if (owner) {
            var src = owner.src;
            if (webView.URL) {
                src = webView.URL.absoluteString;
            }
            owner._onLoadFinished(src);
        }
    };
    WKNavigationDelegateImpl.prototype.webViewDidFailNavigationWithError = function (webView, navigation, error) {
        var owner = this._owner.get();
        if (owner) {
            var src = owner.src;
            if (webView.URL) {
                src = webView.URL.absoluteString;
            }
            owner._onLoadFinished(src, error.localizedDescription);
        }
    };
    WKNavigationDelegateImpl.prototype.webViewDecidePolicyForNavigationActionDecisionHandler = function (webView, navigationAction, decisionHandler) {
        var owner = this._owner.get();
        if (owner && navigationAction.request.URL) {
            var navType = 'other';
            switch (navigationAction.navigationType) {
                case 0:
                    navType = 'linkClicked';
                    break;
                case 1:
                    navType = 'formSubmitted';
                    break;
                case 2:
                    navType = 'backForward';
                    break;
                case 3:
                    navType = 'reload';
                    break;
                case 4:
                    navType = 'formResubmitted';
                    break;
                case -1:
                    navType = 'other';
                    break;
            }
            decisionHandler(1);
            owner._onLoadStarted(navigationAction.request.URL.absoluteString, navType);
        }
    };
    return WKNavigationDelegateImpl;
}(NSObject));
WKNavigationDelegateImpl.ObjCProtocols = [WKNavigationDelegate];
var WKScriptMessageHandlerImpl = (function (_super) {
    __extends(WKScriptMessageHandlerImpl, _super);
    function WKScriptMessageHandlerImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    WKScriptMessageHandlerImpl.new = function () {
        return _super.new.call(this);
    };
    WKScriptMessageHandlerImpl.initWithOwner = function (owner) {
        var handler = WKScriptMessageHandlerImpl.new();
        handler._owner = owner;
        return handler;
    };
    WKScriptMessageHandlerImpl.prototype.userContentControllerDidReceiveScriptMessage = function (userContentController, message) {
        var owner = this._owner.get();
        if (owner) {
            owner.notify({
                eventName: TNSWKWebView.messageHandlerEvent,
                object: observable_1.fromObject({
                    name: message.name,
                    body: message.body
                })
            });
        }
    };
    return WKScriptMessageHandlerImpl;
}(NSObject));
WKScriptMessageHandlerImpl.ObjCProtocols = [WKScriptMessageHandler];
var WKUIDelegateImpl = (function (_super) {
    __extends(WKUIDelegateImpl, _super);
    function WKUIDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    WKUIDelegateImpl.initWithOwner = function (owner) {
        var handler = WKUIDelegateImpl.new();
        handler._owner = owner;
        return handler;
    };
    WKUIDelegateImpl.prototype.webViewCommitPreviewingViewController = function (webView, previewingViewController) {
    };
    WKUIDelegateImpl.prototype.webViewCreateWebViewWithConfigurationForNavigationActionWindowFeatures = function (webView, configuration, navigationAction, windowFeatures) {
        var owner = this._owner.get();
        if (owner) {
            var eventName = void 0;
            switch (navigationAction.navigationType) {
                case 0:
                    eventName = 'linkClicked';
                    break;
                case 1:
                    eventName = 'formSubmitted';
                    break;
                case 2:
                    eventName = 'backForward';
                    break;
                case 3:
                    eventName = 'reload';
                    break;
                case 4:
                    eventName = 'formResubmitted';
                    break;
                case -1:
                    eventName = 'other';
                    break;
            }
            owner.notify({
                eventName: TNSWKWebView.openEvent,
                object: observable_1.fromObject({
                    name: eventName,
                    body: {
                        url: navigationAction.request.URL.absoluteString
                    }
                })
            });
        }
        return owner.ios;
    };
    WKUIDelegateImpl.prototype.webViewDidClose = function (webView) {
        var owner = this._owner.get();
        owner.notify({
            eventName: TNSWKWebView.closedEvent,
            object: observable_1.fromObject({})
        });
    };
    WKUIDelegateImpl.prototype.webViewPreviewingViewControllerForElementDefaultActions = function (webView, elementInfo, previewActions) {
        return null;
    };
    WKUIDelegateImpl.prototype.webViewRunJavaScriptAlertPanelWithMessageInitiatedByFrameCompletionHandler = function (webView, message, frame, completionHandler) {
    };
    WKUIDelegateImpl.prototype.webViewRunJavaScriptConfirmPanelWithMessageInitiatedByFrameCompletionHandler = function (webView, message, frame, completionHandler) {
    };
    WKUIDelegateImpl.prototype.webViewRunJavaScriptTextInputPanelWithPromptDefaultTextInitiatedByFrameCompletionHandler = function (webView, prompt, defaultText, frame, completionHandler) {
    };
    WKUIDelegateImpl.prototype.webViewShouldPreviewElement = function (webView, elementInfo) {
        return false;
    };
    return WKUIDelegateImpl;
}(NSObject));
WKUIDelegateImpl.ObjCProtocols = [WKUIDelegate];
var srcProperty = new view_1.Property({ name: 'src' });
var TNSWKWebView = (function (_super) {
    __extends(TNSWKWebView, _super);
    function TNSWKWebView() {
        var _this = _super.call(this) || this;
        _this._messageHandlers = [];
        var jScript = "var meta = document.createElement('meta'); meta.setAttribute('name', 'viewport'); meta.setAttribute('content', 'initial-scale=1.0'); document.getElementsByTagName('head')[0].appendChild(meta);";
        var wkUScript = WKUserScript.alloc().initWithSourceInjectionTimeForMainFrameOnly(jScript, 0, true);
        _this._scriptMessageHandler = WKScriptMessageHandlerImpl.initWithOwner(new WeakRef(_this));
        _this._userContentController = WKUserContentController.new();
        _this._userContentController.addUserScript(wkUScript);
        var config = WKWebViewConfiguration.new();
        _this._delegate = WKNavigationDelegateImpl.initWithOwner(new WeakRef(_this));
        _this._uiDelegate = WKUIDelegateImpl.initWithOwner(new WeakRef(_this));
        config.preferences.setValueForKey(true, 'allowFileAccessFromFileURLs');
        _this.nativeView = _this._ios = new WKWebView({
            frame: CGRectZero,
            configuration: config
        });
        return _this;
    }
    TNSWKWebView.prototype._onLoadFinished = function (url, error) {
        var args = {
            eventName: TNSWKWebView.loadFinishedEvent,
            object: this,
            url: url,
            navigationType: undefined,
            error: error
        };
        this.notify(args);
    };
    TNSWKWebView.prototype._onLoadStarted = function (url, navigationType) {
        var args = {
            eventName: TNSWKWebView.loadStartedEvent,
            object: this,
            url: url,
            navigationType: navigationType,
            error: undefined
        };
        this.notify(args);
    };
    TNSWKWebView.prototype.stopLoading = function () {
        this._ios.stopLoading();
    };
    TNSWKWebView.prototype.onMeasure = function (widthMeasureSpec, heightMeasureSpec) {
        var nativeView = this.nativeView;
        if (nativeView) {
            var width = view_1.layout.getMeasureSpecSize(widthMeasureSpec);
            var height = view_1.layout.getMeasureSpecSize(heightMeasureSpec);
            this.setMeasuredDimension(width, height);
        }
    };
    TNSWKWebView.prototype[srcProperty.getDefault] = function () {
        return '';
    };
    TNSWKWebView.prototype[srcProperty.setNative] = function (src) {
        this.stopLoading();
        if (src.startsWith('~/') ||
            src.startsWith('http') ||
            src.startsWith('/') ||
            src.startsWith('file:///')) {
            this._loadUrl(src, this.readAccessUrl);
        }
        else {
            this._loadData(src);
        }
    };
    TNSWKWebView.prototype.onLoaded = function () {
        _super.prototype.onLoaded.call(this);
        this._ios.configuration.userContentController = this._userContentController;
        this._ios.navigationDelegate = this._delegate;
        this._ios.UIDelegate = this._uiDelegate;
    };
    TNSWKWebView.prototype.onUnloaded = function () {
        this._ios.navigationDelegate = null;
        this._ios.UIDelegate = null;
        _super.prototype.onUnloaded.call(this);
    };
    Object.defineProperty(TNSWKWebView.prototype, "canGoBack", {
        get: function () {
            return this._ios.canGoBack;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TNSWKWebView.prototype, "canGoForward", {
        get: function () {
            var nativeView = this.nativeView;
            if (nativeView) {
                return nativeView.canGoForward;
            }
            return false;
        },
        enumerable: true,
        configurable: true
    });
    TNSWKWebView.prototype.goBack = function () {
        var nativeView = this.nativeView;
        if (nativeView) {
            nativeView.goBack();
        }
    };
    TNSWKWebView.prototype.goForward = function () {
        var nativeView = this.nativeView;
        if (nativeView) {
            nativeView.goForward();
        }
    };
    TNSWKWebView.prototype._loadData = function (content) {
        this._ios.loadHTMLStringBaseURL(content, NSURL.alloc().initWithString("file:///" + file_system_1.knownFolders.currentApp().path + "/"));
    };
    TNSWKWebView.prototype._loadUrl = function (url, readAccessUrl) {
        var reHttp = new RegExp('^http');
        if (reHttp.test(url)) {
            var myUrl = NSURL.URLWithString(url);
            var myRequest = NSURLRequest.requestWithURL(myUrl);
            this._ios.loadRequest(myRequest);
        }
        else if (url.startsWith('/')) {
            var myUrl = NSURL.URLWithString(encodeURI('file://' + url));
            var myReadAccessUrl = readAccessUrl
                ? NSURL.fileURLWithPath(readAccessUrl)
                : myUrl;
            this._ios.loadFileURLAllowingReadAccessToURL(myUrl, myReadAccessUrl);
        }
        else if (url.startsWith('file:///')) {
            var myUrl = NSURL.URLWithString(encodeURI(url));
            var myReadAccessUrl = NSURL.fileURLWithPath(this.readAccessUrl || url);
            this._ios.loadFileURLAllowingReadAccessToURL(myUrl, myReadAccessUrl);
        }
        else {
            var reTilda = new RegExp('^~/');
            if (reTilda.test(url)) {
                url = fs.path.join(fs.knownFolders.currentApp().path, url.replace(reTilda, ''));
            }
            var myUrl = NSURL.URLWithString(encodeURI('file://' + url));
            var myReadAccessUrl = readAccessUrl
                ? NSURL.fileURLWithPath(readAccessUrl)
                : myUrl;
            this._ios.loadFileURLAllowingReadAccessToURL(myUrl, myReadAccessUrl);
        }
    };
    TNSWKWebView.prototype.reload = function () {
        this._ios.reload();
    };
    TNSWKWebView.prototype.addMessageHandler = function (messageHandlerName) {
        if (this._messageHandlers.indexOf(messageHandlerName) === -1) {
            this._userContentController.addScriptMessageHandlerName(this._scriptMessageHandler, messageHandlerName);
            this._messageHandlers.push(messageHandlerName);
        }
    };
    TNSWKWebView.prototype.removeMessageHandler = function (messageHandlerName) {
        var index = this._messageHandlers.indexOf(messageHandlerName);
        if (index > -1) {
            this._userContentController.removeScriptMessageHandlerForName(messageHandlerName);
            this._messageHandlers.splice(index, 1);
        }
    };
    TNSWKWebView.prototype.evaluateJavaScript = function (javaScriptString, callback) {
        this._ios.evaluateJavaScriptCompletionHandler(javaScriptString, function (res, err) {
            callback(res, err);
        });
    };
    return TNSWKWebView;
}(view_1.View));
TNSWKWebView.messageHandlerEvent = 'message';
TNSWKWebView.loadStartedEvent = 'loadStarted';
TNSWKWebView.loadFinishedEvent = 'loadFinished';
TNSWKWebView.closedEvent = 'closed';
TNSWKWebView.openEvent = 'open';
exports.TNSWKWebView = TNSWKWebView;
srcProperty.register(TNSWKWebView);
//# sourceMappingURL=webkit-webview.ios.js.map